// online-mfcnet.h
// If you want to put this file under the directory, it file is placed in under the "feat" directory.
// Implements like "feat/online-feature.h".
// Implements like "online2/online-nnet2-feature-pipeline.h".

#ifndef ONLINE_FEATURE_MFCNET_H
#define ONLINE_FEATURE_MFCNET_H

#include <atomic>
#include <sstream>

#include <string>

#include "itf/online-feature-itf.h"
#include "mp/sleep_wrapper.hpp"
#include "mp/mfcnet_server.hpp"
#include "mp/ex_mfcnet_server.hpp"
#include "mp/hark-support.h"
//FOR_FILE_MODE//#include "matrix/kaldi-matrix.h"
//FOR_FILE_MODE//#include "feat/online-feature.h"

// for OnlineFeatureInputSettingOptions
//#include "feat/feature-mfcc.h"
//#include "feat/feature-plp.h"
//#include "feat/feature-fbank.h"
//#include "feat/pitch-functions.h"
//#include "online2/online-ivector-feature.h"
#include "online2/online-nnet2-feature-pipeline.h"

namespace kaldi {

class OnlineFeatureMfcnet : public OnlineFeatureInterface {
 public:
  OnlineFeatureMfcnet(kaldi::OnlineNnet2FeaturePipelineInfo *feature_info, const kaldi::Matrix<kaldi::BaseFloat>& feats, std::string id);
  OnlineFeatureMfcnet(kaldi::OnlineNnet2FeaturePipelineInfo *feature_info, std::shared_ptr<mfcnet_server> socket, std::string id);
  OnlineFeatureMfcnet(kaldi::OnlineNnet2FeaturePipelineInfo *feature_info, std::shared_ptr<ex_mfcnet_server> socket, std::string id);

  virtual ~OnlineFeatureMfcnet();

  void Initialize();

  bool is_file_input(){ return (this->mfcnet_socket_.get() == nullptr && this->ex_mfcnet_socket_.get() == nullptr); }
  bool is_mfcnet_input(){ return (this->mfcnet_socket_.get() != nullptr); }
  bool is_ex_mfcnet_input(){ return (this->ex_mfcnet_socket_.get() != nullptr); }

  virtual kaldi::int32 Dim() const;

  virtual kaldi::int32 NumFramesReady() const;

  virtual bool IsLastFrame(kaldi::int32 frame) const;

  kaldi::BaseFloat FrameShiftInSeconds() const { return this->frame_shift_in_seconds_; } // old version is { return 0.01f; }

  virtual void GetFrame(kaldi::int32 frame, kaldi::VectorBase<kaldi::BaseFloat> *feat);

  void MfcnetReceive();
  void ExMfcnetReceive();
  void Receive();

  const SourceInfo& GetSourceInfo(){ return this->source_info_; }
  const UtteranceInfo& GetUtteranceInfo(){ return this->utterance_info_; }

  std::string GetID() { return this->id_; }

 private:
  //kaldi::OnlineFeatureInputSettingOptions opts_;
  kaldi::OnlineNnet2FeaturePipelineInfo *feature_info_;
  std::shared_ptr<mfcnet_server> mfcnet_socket_;
  std::shared_ptr<ex_mfcnet_server> ex_mfcnet_socket_;
  std::string id_;
  std::atomic<std::int32_t> feat_dim_;
  std::atomic<std::int32_t> ivector_dim_;
  std::atomic<std::int32_t> feat_bytes_;
  std::atomic<std::int32_t> num_frames_; // The number of frames that the decoder may read.
  //FOR_FILE_MODE//bool is_file;
  std::atomic<bool> input_finished_;
  kaldi::Matrix<kaldi::BaseFloat> features_;
  kaldi::Matrix<kaldi::BaseFloat> ivector_features_;
  SourceInfo source_info_;
  UtteranceInfo utterance_info_;
  kaldi::BaseFloat frame_shift_in_seconds_;
};

}

#endif // ONLINE_FEATURE_MFCNET_H
