// pidstat.hpp: Display per-process statistics.

#ifndef MP_PIDSTAT_H_
#define MP_PIDSTAT_H_

#ifdef HAVE_LINUX_SCHED_H
#include <linux/sched.h>
#endif // HAVE_SYS_LINUX_SCHED_H

// Check definitions and settings defined in <linux/sched.h>
#ifndef SCHED_NORMAL
#define SCHED_NORMAL  0
#endif
#ifndef SCHED_FIFO
#define SCHED_FIFO  1
#endif
#ifndef SCHED_RR
#define SCHED_RR  2
#endif
#ifndef SCHED_BATCH
#define SCHED_BATCH  3
#endif
// SCHED_ISO is not yet implemented
#ifndef SCHED_IDLE
#define SCHED_IDLE  5
#endif
#ifndef SCHED_DEADLINE
#define SCHED_DEADLINE  6
#endif

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif // HAVE_SYS_PARAM_H

// Check definitions and settings defined in <sys/param.h>
#ifndef _POSIX_ARG_MAX
#define _POSIX_ARG_MAX  128;
#endif
#ifndef LOGIN_NAME_MAX
#define LOGIN_NAME_MAX  32;
#endif

#include "stat_print_base.hpp"

namespace multiplatform {

/*
// ActivitiesFlag
class ActivitiesFlag {
  constexpr static uint64_t P_A_NULL  = 0x00;
  constexpr static uint64_t P_A_CPU   = 0x01;
  constexpr static uint64_t P_A_MEM   = 0x02;
  constexpr static uint64_t P_A_IO    = 0x04;
  constexpr static uint64_t P_A_CTXSW = 0x08;
  constexpr static uint64_t P_A_STACK = 0x10;
  constexpr static uint64_t P_A_KTAB  = 0x20;
  constexpr static uint64_t P_A_RT    = 0x40;
  constexpr static uint64_t P_A_ALL   = P_A_CPU   + P_A_MEM   + P_A_IO    + P_A_CTXSW +
                                        P_A_STACK + P_A_KTAB  + P_A_RT;

  ActivitiesFlag(): flag(P_A_NULL) {  }
  ActivitiesFlag(uint64_t l_flag): flag(l_flag) {  } // for the class test
  ~ActivitiesFlag(){  }

  void set_all(void){ flag = P_A_ALL; }
  void set_cpu(void){ flag |= P_A_CPU; }
  void set_mem(void){ flag |= P_A_MEM; }
  void set_io(void){ flag |= P_A_IO; }
  void set_ctxsw(void){ flag |= P_A_CTXSW; }
  void set_stack(void){ flag |= P_A_STACK; }
  void set_ktab(void){ flag |= P_A_KTAB; }
  void set_rt(void){ flag |= P_A_RT; }

  void unset_all(void){ flag = P_A_NULL; }
  void unset_cpu(void){ flag &= ~P_A_CPU; }
  void unset_mem(void){ flag &= ~P_A_MEM; }
  void unset_io(void){ flag &= ~P_A_IO; }
  void unset_ctxsw(void){ flag &= ~P_A_CTXSW; }
  void unset_stack(void){ flag &= ~P_A_STACK; }
  void unset_ktab(void){ flag &= ~P_A_KTAB; }
  void unset_rt(void){ flag &= ~P_A_RT; }

  bool check_cpu(void){ return ((flag & P_A_CPU) == P_A_CPU); }
  bool check_mem(void){ return ((flag & P_A_MEM) == P_A_MEM); }
  bool check_io(void){ return ((flag & P_A_IO) == P_A_IO); }
  bool check_ctxsw(void){ return ((flag & P_A_CTXSW) == P_A_CTXSW); }
  bool check_stack(void){ return ((flag & P_A_STACK) == P_A_STACK); }
  bool check_ktab(void){ return ((flag & P_A_KTAB) == P_A_KTAB); }
  bool check_rt(void){ return ((flag & P_A_RT) == P_A_RT); }

  void set_all(uint64_t &l_flag){ l_flag = P_A_ALL; }
  void set_cpu(uint64_t &l_flag){ l_flag |= P_A_CPU; }
  void set_mem(uint64_t &l_flag){ l_flag |= P_A_MEM; }
  void set_io(uint64_t &l_flag){ l_flag |= P_A_IO; }
  void set_ctxsw(uint64_t &l_flag){ l_flag |= P_A_CTXSW; }
  void set_stack(uint64_t &l_flag){ l_flag |= P_A_STACK; }
  void set_ktab(uint64_t &l_flag){ l_flag |= P_A_KTAB; }
  void set_rt(uint64_t &l_flag){ l_flag |= P_A_RT; }

  void unset_all(uint64_t &l_flag){ l_flag = P_A_NULL; }
  void unset_cpu(uint64_t &l_flag){ l_flag &= ~P_A_CPU; }
  void unset_mem(uint64_t &l_flag){ l_flag &= ~P_A_MEM; }
  void unset_io(uint64_t &l_flag){ l_flag &= ~P_A_IO; }
  void unset_ctxsw(uint64_t &l_flag){ l_flag &= ~P_A_CTXSW; }
  void unset_stack(uint64_t &l_flag){ l_flag &= ~P_A_STACK; }
  void unset_ktab(uint64_t &l_flag){ l_flag &= ~P_A_KTAB; }
  void unset_rt(uint64_t &l_flag){ l_flag &= ~P_A_RT; }

  bool check_cpu(uint64_t l_flag){ return ((l_flag & P_A_CPU) == P_A_CPU); }
  bool check_mem(uint64_t l_flag){ return ((l_flag & P_A_MEM) == P_A_MEM); }
  bool check_io(uint64_t l_flag){ return ((l_flag & P_A_IO) == P_A_IO); }
  bool check_ctxsw(uint64_t l_flag){ return ((l_flag & P_A_CTXSW) == P_A_CTXSW); }
  bool check_stack(uint64_t l_flag){ return ((l_flag & P_A_STACK) == P_A_STACK); }
  bool check_ktab(uint64_t l_flag){ return ((l_flag & P_A_KTAB) == P_A_KTAB); }
  bool check_rt(uint64_t l_flag){ return ((l_flag & P_A_RT) == P_A_RT); }

  uint64_t flag;
};

// TASK/CHILD
class TaskFlag {
  constexpr static uint64_t P_NULL  = 0x00;
  constexpr static uint64_t P_TASK  = 0x01;
  constexpr static uint64_t P_CHILD = 0x02;

  TaskFlag(): flag(P_NULL) {  }
  TaskFlag(uint64_t l_flag): flag(l_flag) {  } // for the class test
  ~TaskFlag(){  }

  //void set_all(void){ flag = P_ALL; }
  void set_task(void){ flag |= P_TASK; }
  void set_child(void){ flag |= P_CHILD; }

  //void unset_all(void){ flag = P_NULL; }
  void unset_task(void){ flag &= ~P_TASK; }
  void unset_child(void){ flag &= ~P_CHILD; }

  bool check_task(void){ return ((flag & P_TASK) == P_TASK); }
  bool check_child(void){ return ((flag & P_CHILD) == P_CHILD); }

  //void set_all(uint64_t &l_flag){ l_flag = P_ALL; }
  void set_task(uint64_t &l_flag){ l_flag |= P_TASK; }
  void set_child(uint64_t &l_flag){ l_flag |= P_CHILD; }

  //void unset_all(uint64_t &l_flag){ l_flag = P_NULL; }
  void unset_task(uint64_t &l_flag){ l_flag &= ~P_TASK; }
  void unset_child(uint64_t &l_flag){ l_flag &= ~P_CHILD; }

  bool check_task(uint64_t l_flag){ return ((l_flag & P_TASK) == P_TASK); }
  bool check_child(uint64_t l_flag){ return ((l_flag & P_CHILD) == P_CHILD); }

  uint64_t flag;
}

class PIDFlag {
  constexpr static uint64_t P_F_NULL       = 0x0000;
  constexpr static uint64_t P_D_PID        = 0x0001;
  constexpr static uint64_t P_D_ALL_PID    = 0x0002;
  constexpr static uint64_t P_F_IRIX_MODE  = 0x0004;
  constexpr static uint64_t P_F_COMMSTR    = 0x0008;
  constexpr static uint64_t P_D_ACTIVE_PID = 0x0010;
  constexpr static uint64_t P_D_TID        = 0x0020;
  constexpr static uint64_t P_D_ONELINE    = 0x0040;
  constexpr static uint64_t P_D_CMDLINE    = 0x0080;
  constexpr static uint64_t P_D_USERNAME   = 0x0100;
  constexpr static uint64_t P_F_USERSTR    = 0x0200;
  constexpr static uint64_t P_F_PROCSTR    = 0x0400;
  constexpr static uint64_t P_D_UNIT       = 0x0800;
  constexpr static uint64_t P_D_SEC_EPOCH  = 0x1000;
  constexpr static uint64_t P_F_EXEC_PGM   = 0x2000;

  PIDFlag(): flag(P_F_NULL) {  }
  PIDFlag(uint64_t l_flag): flag(l_flag) {  } // for the class test
  ~PIDFlag(){  }

  //void set_all(void){ flag = P_F_ALL; }
  void set_pid(void){ flag |= P_D_PID; }
  void set_all_pid(void){ flag |= P_D_ALL_PID; }
  void set_irix_mode(void){ flag |= P_F_IRIX_MODE; }
  void set_commstr(void){ flag |= P_F_COMMSTR; }
  void set_active_pid(void){ flag |= P_D_ACTIVE_PID; }
  void set_tid(void){ flag |= P_D_TID; }
  void set_oneline(void){ flag |= P_D_ONELINE; }
  void set_cmdline(void){ flag |= P_D_CMDLINE; }
  void set_username(void){ flag |= P_D_USERNAME; }
  void set_userstr(void){ flag |= P_F_USERSTR; }
  void set_procstr(void){ flag |= P_F_PROCSTR; }
  void set_unit(void){ flag |= P_D_UNIT; }
  void set_sec_epoch(void){ flag |= P_D_SEC_EPOCH; }
  void set_exec_pgm(void){ flag |= P_F_EXEC_PGM; }

  //void unset_all(void){ flag = P_F_NULL; }
  void unset_pid(void){ flag &= ~P_D_PID; }
  void unset_all_pid(void){ flag &= ~P_D_ALL_PID; }
  void unset_irix_mode(void){ flag &= ~P_F_IRIX_MODE; }
  void unset_commstr(void){ flag &= ~P_F_COMMSTR; }
  void unset_active_pid(void){ flag &= ~P_D_ACTIVE_PID; }
  void unset_tid(void){ flag &= ~P_D_TID; }
  void unset_oneline(void){ flag &= ~P_D_ONELINE; }
  void unset_cmdline(void){ flag &= ~P_D_CMDLINE; }
  void unset_username(void){ flag &= ~P_D_USERNAME; }
  void unset_userstr(void){ flag &= ~P_F_USERSTR; }
  void unset_procstr(void){ flag &= ~P_F_PROCSTR; }
  void unset_unit(void){ flag &= ~P_D_UNIT; }
  void unset_sec_epoch(void){ flag &= ~P_D_SEC_EPOCH; }
  void unset_exec_pgm(void){ flag &= ~P_F_EXEC_PGM; }

  bool check_pid(void){ return ((flag & P_D_PID) == P_D_PID); }
  bool check_all_pid(void){ return ((flag & P_D_ALL_PID) == P_D_ALL_PID); }
  bool check_irix_mode(void){ return ((flag & P_F_IRIX_MODE) == P_F_IRIX_MODE); }
  bool check_commstr(void){ return ((flag & P_F_COMMSTR) == P_F_COMMSTR); }
  bool check_active_pid(void){ return ((flag & P_D_ACTIVE_PID) == P_D_ACTIVE_PID); }
  bool check_tid(void){ return ((flag & P_D_TID) == P_D_TID); }
  bool check_oneline(void){ return ((flag & P_D_ONELINE) == P_D_ONELINE); }
  bool check_cmdline(void){ return ((flag & P_D_CMDLINE) == P_D_CMDLINE); }
  bool check_username(void){ return ((flag & P_D_USERNAME) == P_D_USERNAME); }
  bool check_userstr(void){ return ((flag & P_F_USERSTR) == P_F_USERSTR); }
  bool check_procstr(void){ return ((flag & P_F_PROCSTR) == P_F_PROCSTR); }
  bool check_unit(void){ return ((flag & P_D_UNIT) == P_D_UNIT); }
  bool check_sec_epoch(void){ return ((flag & P_D_SEC_EPOCH) == P_D_SEC_EPOCH); }
  bool check_exec_pgm(void){ return ((flag & P_F_EXEC_PGM) == P_F_EXEC_PGM); }

  //void set_all(uint64_t &l_flag){ l_flag = P_F_ALL; }
  void set_pid(uint64_t &l_flag){ l_flag |= P_D_PID; }
  void set_all_pid(uint64_t &l_flag){ l_flag |= P_D_ALL_PID; }
  void set_irix_mode(uint64_t &l_flag){ l_flag |= P_F_IRIX_MODE; }
  void set_commstr(uint64_t &l_flag){ l_flag |= P_F_COMMSTR; }
  void set_active_pid(uint64_t &l_flag){ l_flag |= P_D_ACTIVE_PID; }
  void set_tid(uint64_t &l_flag){ l_flag |= P_D_TID; }
  void set_oneline(uint64_t &l_flag){ l_flag |= P_D_ONELINE; }
  void set_cmdline(uint64_t &l_flag){ l_flag |= P_D_CMDLINE; }
  void set_username(uint64_t &l_flag){ l_flag |= P_D_USERNAME; }
  void set_userstr(uint64_t &l_flag){ l_flag |= P_F_USERSTR; }
  void set_procstr(uint64_t &l_flag){ l_flag |= P_F_PROCSTR; }
  void set_unit(uint64_t &l_flag){ l_flag |= P_D_UNIT; }
  void set_sec_epoch(uint64_t &l_flag){ l_flag |= P_D_SEC_EPOCH; }
  void set_exec_pgm(uint64_t &l_flag){ l_flag |= P_F_EXEC_PGM; }

  //void unset_all(uint64_t &l_flag){ l_flag = P_F_NULL; }
  void unset_pid(uint64_t &l_flag){ l_flag &= ~P_D_PID; }
  void unset_all_pid(uint64_t &l_flag){ l_flag &= ~P_D_ALL_PID; }
  void unset_irix_mode(uint64_t &l_flag){ l_flag &= ~P_F_IRIX_MODE; }
  void unset_commstr(uint64_t &l_flag){ l_flag &= ~P_F_COMMSTR; }
  void unset_active_pid(uint64_t &l_flag){ l_flag &= ~P_D_ACTIVE_PID; }
  void unset_tid(uint64_t &l_flag){ l_flag &= ~P_D_TID; }
  void unset_oneline(uint64_t &l_flag){ l_flag &= ~P_D_ONELINE; }
  void unset_cmdline(uint64_t &l_flag){ l_flag &= ~P_D_CMDLINE; }
  void unset_username(uint64_t &l_flag){ l_flag &= ~P_D_USERNAME; }
  void unset_userstr(uint64_t &l_flag){ l_flag &= ~P_F_USERSTR; }
  void unset_procstr(uint64_t &l_flag){ l_flag &= ~P_F_PROCSTR; }
  void unset_unit(uint64_t &l_flag){ l_flag &= ~P_D_UNIT; }
  void unset_sec_epoch(uint64_t &l_flag){ l_flag &= ~P_D_SEC_EPOCH; }
  void unset_exec_pgm(uint64_t &l_flag){ l_flag &= ~P_F_EXEC_PGM; }

  bool check_pid(uint64_t l_flag){ return ((l_flag & P_D_PID) == P_D_PID); }
  bool check_all_pid(uint64_t l_flag){ return ((l_flag & P_D_ALL_PID) == P_D_ALL_PID); }
  bool check_irix_mode(uint64_t l_flag){ return ((l_flag & P_F_IRIX_MODE) == P_F_IRIX_MODE); }
  bool check_commstr(uint64_t l_flag){ return ((l_flag & P_F_COMMSTR) == P_F_COMMSTR); }
  bool check_active_pid(uint64_t l_flag){ return ((l_flag & P_D_ACTIVE_PID) == P_D_ACTIVE_PID); }
  bool check_tid(uint64_t l_flag){ return ((l_flag & P_D_TID) == P_D_TID); }
  bool check_oneline(uint64_t l_flag){ return ((l_flag & P_D_ONELINE) == P_D_ONELINE); }
  bool check_cmdline(uint64_t l_flag){ return ((l_flag & P_D_CMDLINE) == P_D_CMDLINE); }
  bool check_username(uint64_t l_flag){ return ((l_flag & P_D_USERNAME) == P_D_USERNAME); }
  bool check_userstr(uint64_t l_flag){ return ((l_flag & P_F_USERSTR) == P_F_USERSTR); }
  bool check_procstr(uint64_t l_flag){ return ((l_flag & P_F_PROCSTR) == P_F_PROCSTR); }
  bool check_unit(uint64_t l_flag){ return ((l_flag & P_D_UNIT) == P_D_UNIT); }
  bool check_sec_epoch(uint64_t l_flag){ return ((l_flag & P_D_SEC_EPOCH) == P_D_SEC_EPOCH); }
  bool check_exec_pgm(uint64_t l_flag){ return ((l_flag & P_F_EXEC_PGM) == P_F_EXEC_PGM); }

  void PRINT_ID_HDR(_timestamp_){
    printf("\n%-11s", _timestamp_);
    if(check_username()){ printf("     USER"); }
    else{                 printf("   UID"); }
    if(check_tid()){ printf("      TGID       TID"); }
    else{            printf("       PID"); }
  }

  void PRINT_ID_HDR(_timestamp_, uint64_t l_flag){
    printf("\n%-11s", _timestamp_);
    if(check_username(l_flag)){ printf("     USER"); }
    else{                       printf("   UID"); }
    if(check_tid(l_flag)){ printf("      TGID       TID"); }
    else{                  printf("       PID"); }
  }

  uint64_t flag;
};

// Per-process flags
class PerProcessFlag {
  constexpr static uint64_t F_NULL          = 0x00;
  constexpr static uint64_t F_NO_PID_IO     = 0x01;
  constexpr static uint64_t F_NO_PID_FD     = 0x02;
  constexpr static uint64_t F_PID_DISPLAYED = 0x04;

  PerProcessFlag(): flag(F_NULL) {  }
  PerProcessFlag(uint64_t l_flag): flag(l_flag) {  } // for the class test
  ~PerProcessFlag(){  }

  //void set_all(void){ flag = F_ALL; }
  void set_no_pid_io(void){ flag |= F_NO_PID_IO; }
  void set_no_pid_fd(void){ flag |= F_NO_PID_FD; }
  void set_pid_displayed(void){ flag |= F_PID_DISPLAYED; }

  //void unset_all(void){ flag = F_NULL; }
  void unset_no_pid_io(void){ flag &= ~F_NO_PID_IO; }
  void unset_no_pid_fd(void){ flag &= ~F_NO_PID_FD; }
  void unset_pid_displayed(void){ flag &= ~F_PID_DISPLAYED; }

  bool check_no_pid_io(void){ return ((flag & F_NO_PID_IO) == F_NO_PID_IO); }
  bool check_no_pid_fd(void){ return ((flag & F_NO_PID_FD) == F_NO_PID_FD); }
  bool check_pid_displayed(void){ return ((flag & F_PID_DISPLAYED) == F_PID_DISPLAYED); }

  //void set_all(uint64_t &l_flag){ l_flag = F_ALL; }
  void set_no_pid_io(uint64_t &l_flag){ l_flag |= F_NO_PID_IO; }
  void set_no_pid_fd(uint64_t &l_flag){ l_flag |= F_NO_PID_FD; }
  void set_pid_displayed(uint64_t &l_flag){ l_flag |= F_PID_DISPLAYED; }

  //void unset_all(uint64_t &l_flag){ l_flag = P_F_NULL; }
  void unset_no_pid_io(uint64_t &l_flag){ l_flag &= ~F_NO_PID_IO; }
  void unset_no_pid_fd(uint64_t &l_flag){ l_flag &= ~F_NO_PID_FD; }
  void unset_pid_displayed(uint64_t &l_flag){ l_flag &= ~F_PID_DISPLAYED; }

  bool check_no_pid_io(uint64_t l_flag){ return ((l_flag & F_NO_PID_IO) == F_NO_PID_IO); }
  bool check_no_pid_fd(uint64_t l_flag){ return ((l_flag & F_NO_PID_FD) == F_NO_PID_FD); }
  bool check_pid_displayed(uint64_t l_flag){ return ((l_flag & F_PID_DISPLAYED) == F_PID_DISPLAYED); }

  uint64_t flag;
};
*/

// ActivitiesFlag
constexpr static unsigned int P_A_CPU   = 0x01;
constexpr static unsigned int P_A_MEM   = 0x02;
constexpr static unsigned int P_A_IO    = 0x04;
constexpr static unsigned int P_A_CTXSW = 0x08;
constexpr static unsigned int P_A_STACK = 0x10;
constexpr static unsigned int P_A_KTAB  = 0x20;
constexpr static unsigned int P_A_RT    = 0x40;

constexpr bool DISPLAY_CPU(unsigned int l_flag){ return ((l_flag & P_A_CPU) == P_A_CPU); }
constexpr bool DISPLAY_MEM(unsigned int l_flag){ return ((l_flag & P_A_MEM) == P_A_MEM); }
constexpr bool DISPLAY_IO(unsigned int l_flag){ return ((l_flag & P_A_IO) == P_A_IO); }
constexpr bool DISPLAY_CTXSW(unsigned int l_flag){ return ((l_flag & P_A_CTXSW) == P_A_CTXSW); }
constexpr bool DISPLAY_STACK(unsigned int l_flag){ return ((l_flag & P_A_STACK) == P_A_STACK); }
constexpr bool DISPLAY_KTAB(unsigned int l_flag){ return ((l_flag & P_A_KTAB) == P_A_KTAB); }
constexpr bool DISPLAY_RT(unsigned int l_flag){ return ((l_flag & P_A_RT) == P_A_RT); }

// TASK/CHILD
constexpr static unsigned int P_NULL   = 0x00;
constexpr static unsigned int P_TASK   = 0x01;
constexpr static unsigned int P_CHILD  = 0x02;

constexpr bool DISPLAY_TASK_STATS(unsigned int l_flag){ return ((l_flag & P_TASK) == P_TASK); }
constexpr bool DISPLAY_CHILD_STATS(unsigned int l_flag){ return ((l_flag & P_CHILD) == P_CHILD); }

constexpr static unsigned int P_D_PID        = 0x0001;
constexpr static unsigned int P_D_ALL_PID    = 0x0002;
constexpr static unsigned int P_F_IRIX_MODE  = 0x0004;
constexpr static unsigned int P_F_COMMSTR    = 0x0008;
constexpr static unsigned int P_D_ACTIVE_PID = 0x0010;
constexpr static unsigned int P_D_TID        = 0x0020;
constexpr static unsigned int P_D_ONELINE    = 0x0040;
constexpr static unsigned int P_D_CMDLINE    = 0x0080;
constexpr static unsigned int P_D_USERNAME   = 0x0100;
constexpr static unsigned int P_F_USERSTR    = 0x0200;
constexpr static unsigned int P_F_PROCSTR    = 0x0400;
constexpr static unsigned int P_D_UNIT       = 0x0800;
constexpr static unsigned int P_D_SEC_EPOCH  = 0x1000;
constexpr static unsigned int P_F_EXEC_PGM   = 0x2000;

constexpr bool DISPLAY_PID(unsigned int l_flag){ return ((l_flag & P_D_PID) == P_D_PID); }
constexpr bool DISPLAY_ALL_PID(unsigned int l_flag){ return ((l_flag & P_D_ALL_PID) == P_D_ALL_PID); }
constexpr bool IRIX_MODE_OFF(unsigned int l_flag){ return ((l_flag & P_F_IRIX_MODE) == P_F_IRIX_MODE); }
constexpr bool COMMAND_STRING(unsigned int l_flag){ return ((l_flag & P_F_COMMSTR) == P_F_COMMSTR); }
constexpr bool DISPLAY_ACTIVE_PID(unsigned int l_flag){ return ((l_flag & P_D_ACTIVE_PID) == P_D_ACTIVE_PID); }
constexpr bool DISPLAY_TID(unsigned int l_flag){ return ((l_flag & P_D_TID) == P_D_TID); }
constexpr bool DISPLAY_ONELINE(unsigned int l_flag){ return ((l_flag & P_D_ONELINE) == P_D_ONELINE); }
constexpr bool DISPLAY_CMDLINE(unsigned int l_flag){ return ((l_flag & P_D_CMDLINE) == P_D_CMDLINE); }
constexpr bool DISPLAY_USERNAME(unsigned int l_flag){ return ((l_flag & P_D_USERNAME) == P_D_USERNAME); }
constexpr bool USER_STRING(unsigned int l_flag){ return ((l_flag & P_F_USERSTR) == P_F_USERSTR); }
constexpr bool PROCESS_STRING(unsigned int l_flag){ return ((l_flag & P_F_PROCSTR) == P_F_PROCSTR); }
constexpr bool DISPLAY_UNIT(unsigned int l_flag){ return ((l_flag & P_D_UNIT) == P_D_UNIT); }
constexpr bool PRINT_SEC_EPOCH(unsigned int l_flag){ return ((l_flag & P_D_SEC_EPOCH) == P_D_SEC_EPOCH); }
constexpr bool EXEC_PGM(unsigned int l_flag){ return ((l_flag & P_F_EXEC_PGM) == P_F_EXEC_PGM); }

// Per-process flags
constexpr unsigned int F_NO_PID_IO     = 0x01;
constexpr unsigned int F_NO_PID_FD     = 0x02;
constexpr unsigned int F_PID_DISPLAYED = 0x04;

constexpr bool NO_PID_IO(unsigned int l_flag){ return ((l_flag & F_NO_PID_IO) == F_NO_PID_IO); }
constexpr bool NO_PID_FD(unsigned int l_flag){ return ((l_flag & F_NO_PID_FD) == F_NO_PID_FD); }
constexpr bool IS_PID_DISPLAYED(unsigned int l_flag){ return ((l_flag & F_PID_DISPLAYED) == F_PID_DISPLAYED); }

constexpr void PRINT_ID_HDR(char *l_timestamp, unsigned int l_flag){
  printf("\n%-11s", l_timestamp);
  printf(DISPLAY_USERNAME(l_flag) ? "     USER" : "   UID");
  printf(DISPLAY_TID(l_flag) ? "      TGID       TID" : "       PID");
}



constexpr static unsigned int POLICY_NORMAL   = SCHED_NORMAL;
constexpr static unsigned int POLICY_FIFO     = SCHED_FIFO;
constexpr static unsigned int POLICY_RR       = SCHED_RR;
constexpr static unsigned int POLICY_BATCH    = SCHED_BATCH;
constexpr static unsigned int POLICY_IDLE     = SCHED_IDLE;
constexpr static unsigned int POLICY_DEADLINE = SCHED_DEADLINE;

constexpr const char * GET_POLICY(unsigned int p){
  return (p == POLICY_NORMAL   ? "NORMAL" :
         (p == POLICY_FIFO     ? "FIFO" :
         (p == POLICY_RR       ? "RR" :
         (p == POLICY_BATCH    ? "BATCH" :
         (p == POLICY_IDLE     ? "IDLE" :
         (p == POLICY_DEADLINE ? "DEADLN" :
         "?"))))));
}


constexpr static size_t MAX_COMM_LEN = _POSIX_ARG_MAX;
constexpr static size_t MAX_USER_LEN = LOGIN_NAME_MAX;


struct pid_stats {
  uint64_t read_bytes;
  uint64_t write_bytes;
  uint64_t cancelled_write_bytes;
  uint64_t blkio_swapin_delays;
  uint64_t minflt;
  uint64_t cminflt;
  uint64_t majflt;
  uint64_t cmajflt;
  uint64_t utime;
  int64_t  cutime;
  uint64_t stime;
  int64_t  cstime;
  uint64_t gtime;
  int64_t  cgtime;
  uint64_t wtime;
  uint64_t vsz;
  uint64_t rss;
  uint64_t nvcsw;
  uint64_t nivcsw;
  uint64_t stack_size;
  uint64_t stack_ref;
  unsigned int  processor;
  unsigned int  priority;
  unsigned int  policy;
  unsigned int  threads;
  unsigned int  fd_nr;
/*
  pid_stats(){
    read_bytes = 0; write_bytes = 0; cancelled_write_bytes = 0;
    blkio_swapin_delays = 0; minflt = 0; cminflt = 0; majflt = 0; cmajflt = 0;
    utime = 0; cutime = 0; stime = 0; cstime = 0; gtime = 0; cgtime = 0; wtime = 0;
    vsz = 0; rss = 0; nvcsw = 0; nivcsw = 0; stack_size = 0; stack_ref = 0;
    processor = 0; priority = 0; policy = 0; threads = 0; fd_nr = 0;
  }
  ~pid_stats(){}
*/
};

struct st_pid {
  uint64_t total_vsz;
  uint64_t total_rss;
  uint64_t total_stack_size;
  uint64_t total_stack_ref;
  uint64_t total_threads;
  uint64_t total_fd_nr;
  pid_t    pid;
  uid_t    uid;
  int      exist;  // TRUE if PID exists
  unsigned int flags;
  unsigned int rt_asum_count;
  unsigned int rc_asum_count;
  unsigned int uc_asum_count;
  unsigned int tf_asum_count;
  unsigned int sk_asum_count;
  unsigned int delay_asum_count;
  struct pid_stats *pstats[3];
  struct st_pid    *tgid;  // If current task is a TID, pointer to its TGID. NULL otherwise.
  struct st_pid    *next;
  char     comm[MAX_COMM_LEN];
  char     cmdline[MAX_COMM_LEN];
};

/*
class pid_lists {
public:
  struct st_pid *pid_list = NULL;
  struct st_pid **plist;

  pid_lists(){  }
  ~pid_lists(){  }

  void sfree_pid(int force);
  void set_pid_nonexistent(struct st_pid *plist);
  struct st_pid *add_list_pid(struct st_pid **plist, pid_t pid, pid_t tgid);

private:
  int dummy;
};
*/

} // namespace multiplatform

#endif // MP_PIDSTAT_H_

